@@ -22,8 +22,10 @@ use Net::OAI::ListSets;
 use Net::OAI::Record::Header;
 use Net::OAI::Record::OAI_DC;
 
-our $VERSION = '1.16_10';
+our $VERSION = '1.16_11';
 our $DEBUG = 0;
+# compatibility mode for metadataHandler
+our $OLDmetadataHandler = 0;
 
 =head1 NAME
 
@@ -386,8 +388,17 @@ If you would like to use your own metadata handler then you can specify
 the package name of the handler as the C<metadataHandler> (will be exposed
 to events below the C<metadata> element) or C<recordHandler> (will be
 exposed to the C<record> element and all its children) parameter, passing
-either the name of the class as string or an already instantiated object of 
-that class which will be reused for all records. 
+either
+
+=over 4
+
+=item the name of the class as string, in that case a new instance
+will be created for any OAI record encountered or
+
+=item an already instantiated object of that class which will 
+be reused for all records. 
+
+=back
 
     my $records = $harvester->listRecords(
 	metadataPrefix	=> 'mods',
@@ -399,13 +410,18 @@ that class which will be reused for all records.
 	# $metadata will be a MODS::Handler object
     }
 
-If you want to automatically handle resumption tokens you can with 
-the listAllRecords() method.
+If you want to automatically handle resumption tokens you can achieve
+this with the listAllRecords() method. In this case the C<next()> 
+method transparently causes the next response to be fetched from
+the repository if the current response ran out of records and 
+contained a resumptionToken.
 
 If you prefer you can handle resumption tokens yourself with a 
 loop, and the resumptionToken() method. You might want to do this
 if you are working with a repository that wants you to wait between
-requests.
+requests or if connectivity problems become an issue during particulary
+long harvesting runs and you want to implement a retransmission
+strategy for failing requests.
 
     my $records = $harvester->listRecords( metadataPrefix => 'oai_dc' );
     my $responseDate = $records->responseDate();
@@ -479,8 +495,16 @@ sub listRecords {
 
 =head2 listAllRecords() 
 
-Does exactly what listRecords() does except it will automatically
-submit resumption tokens as needed.
+Does exactly what listRecords() does except the C<next()> method 
+will automatically submit resumption tokens as needed.
+
+    my $records = $harvester->listAllRecords( metadataPrefix => 'oai_dc' );
+
+    while ( my $record = $records->next() ) { # a Net::OAI::Record object until undef
+	my $metadata = $record->metadata();
+	# do interesting stuff here 
+    }
+
 
 =cut
 
@@ -508,7 +532,7 @@ L<Net::OAI::ListIdentifiers> object.
 	print "identifier: ", $header->identifier(), "\n";
     }
 
-If you want to automtically handle resumption tokens use listAllIdentifiers().
+If you want to automatically handle resumption tokens use listAllIdentifiers().
 If you are working with a repository that encourages pauses between requests
 you can handle the tokens yourself using the technique described above
 in listRecords().
@@ -547,7 +571,7 @@ sub listIdentifiers {
 
 =head2 listAllIdentifiers()
 
-Does exactly what listIdentifiers() does except it will automatically 
+Does exactly what listIdentifiers() does except C<next()> will automatically 
 submit resumption tokens as needed.
 
 =cut
@@ -560,6 +584,7 @@ sub listAllIdentifiers {
     return( $list );
 }
 
+
 =head2 listSets()
 
 listSets() takes an optional C<resumptionToken> parameter, and returns a 
@@ -829,6 +854,11 @@ Create common handlers for other metadata formats (MARC, qualified DC, etc).
 
 =item *
 
+Or at least provide a generic record handler as fallback, since using
+L<Net::OAI::Record::OAI_DC> makes absolutely no sense except for ... oai_dc records.
+
+=item *
+
 Selectively load Net::OAI::* classes as needed, rather than getting all of them 
 at once at the beginning of Net::OAI::Harvester.
 
