@@ -29,6 +29,9 @@ sub new {
 
 =head2 header()
 
+Returns the C<Net::OAI::Header> object for the OAI header element that
+accompanied the record.
+
 =cut
 
 sub header {
@@ -37,6 +40,18 @@ sub header {
 }
 
 =head2 metadata()
+=head2 recorddata()
+
+Returns the object (SAX Handler!) used or created by the C<metadataHandler> 
+rsp. C<recordHandler> filter class at the moment the OAI::Record was created, 
+namely right after seeing the closing OAI record tag. Or - in the case of
+a C<Net::OAI::Record> provided by the C<next()> method of C<ListRecords>,
+a clone created by means of L<Storable>.
+
+Will be C<undef> when no corresponding option was provided.
+
+Access to the actual data if desired has to be provided by the Handler class.
+Note that in the case of deleted records the 
 
 =cut 
 
@@ -46,9 +61,6 @@ sub metadata {
     return $self->{ metadata } || undef;
 }
 
-=head2 recorddata()
-
-=cut 
 
 sub recorddata {
     my $self = shift;
