@@ -3,7 +3,7 @@ package Net::OAI::Record::Header;
 use strict;
 use base qw( XML::SAX::Base );
 use Carp qw( carp );
-our $VERSION = 'v1.016.010';
+our $VERSION = 'v1.016.011';
 
 
 =head1 NAME
@@ -31,6 +31,10 @@ argument to the constructor of this Header class and result in
 forwarding all events in the C<record> (including C<record> itself) to 
 the handler specified, not only those from C<metadata> children.
 
+In case of compatibility issues of Filters written for older veresions
+you might set C<$Net::OAI::Harvester::OLDmetadataHandler = 1>,
+in which case the metadataHandler Option behaves like a recordHandler.
+
 The SAX filter implemented by this class purposefully does not generate 
 any start_document() or end_document() events.
 
