@@ -3,7 +3,8 @@ package Net::OAI::Record::NamespaceFilter;
 use strict;
 use base qw( XML::SAX::Base );
 use Storable;
-our $VERSION = 'v1.016.10';
+use Carp qw( carp croak );
+our $VERSION = 'v1.016.11';
 
 our $AUTOLOAD;
 
@@ -13,6 +14,27 @@ Net::OAI::Record::NamespaceFilter - general filter class
 
 =head1 SYNOPSIS
 
+ $plug = Net::OAI::Record::NamespaceFilter(); # Noop
+
+ $multihandler = Net::OAI::Record::NamespaceFilter(
+    'http://www.openarchives.org/OAI/2.0/oai_dc/' => 'Net::OAI::Record::OAI_DC',
+    'http://www.openarchives.org/OAI/2.0/provenance' => 'My::ProvenanceHandler'
+   );
+
+ $saxfilter = new SOME_SAX_Filter;
+ ...
+ $filter = Net::OAI::Record::NamespaceFilter(
+    '*' => $saxfilter, # '*' any namespace
+   );
+
+ $filter = Net::OAI::Record::NamespaceFilter(
+   '*' => sub { my $x = ""; 
+                return XML::SAX::Writer->new(Output => \$x);
+              };
+  );
+
+
+
 =head1 DESCRIPTION
 
 It will forward any element belonging to a namespace from this list 
@@ -23,7 +45,7 @@ C<metadataHandler> or C<recordHandler>.
 This SAX filter takes a hashref C<namespaces> as argument, with namespace 
 URIs for keys ('*' for "any") and either 
 
-over 4
+=over 4
 
 =item undef
 
@@ -32,16 +54,16 @@ Matching elements and their subelements are suppressed.
 If the list of namespaces ist empty or C<undefined> is connected to 
 the filter, it effectively acts as a plug to Net::OAI::Harvester. This
 might come handy if you are planning to get to the raw result by other
-means, e.g. by tapping the user agent or accessing the result's xml() method:
+means, e.g. by tapping the user agent or accessing the result's xml()
+method:
 
- $plug = Net::OAI::Harvester::Record::NamespaceFilter();
+ $plug = Net::OAI::Record::NamespaceFilter();
  $harvester = Net::OAI::Harvester->new( [
      baseURL => ...,
-     recordHandler => $plug,
      ] );
 
- my $unparsed;
- open (my $TAP, ">", \$unparsed);
+ $unparsed = "";
+ open ($TAP, ">", \$unparsed);
  $harvester->userAgent()->add_handler(response_data => sub { 
         my($response, $ua, $h, $data) = @_;
         print $TAP $data;
@@ -58,39 +80,54 @@ means, e.g. by tapping the user agent or accessing the result's xml() method:
 
 =item a class name of a SAX filter
 
-For any record element of the OAI response a new instance 
-is created.
+As usual for any record element of the OAI response a new instance is created.
+
+  # end_document() of instances of MyWriter returns something meaningful...
+  $consumer = Net::OAI::Record::NamespaceFilter->new('*'=> 'MyWriter');
+
+  $filter = Net::OAI::Record::NamespaceFilter(
+      '*' => $consumer
+    );
+ 
+  $list = $harvester->listAllRecords( 
+     metadataPrefix  => 'oai_dc',
+     recordHandler => $filter,
+   );
+
+  while( $r = $list->next() ) {
+     next if $r->status() eq "deleted";
+     $xmlstringref = $r->recorddata()->result('*');
+     ...
+  };
+
 
 
 =item a code reference for an constructor
 
 Must return a SAX filter ready to accept a new document.
 
-The following example returns a text representation of each single
+The following example returns a string serialization for each single
 record:
 
  # end_document() events will return \$x
- my $constructor = sub { my $x = ""; 
-                         return XML::SAX::Writer->new(Output => \$x);
-                       };
- $my harvester = Net::OAI::Harvester->new( [
-     baseURL => ...,
-     ] );
-
- my $filter = Net::OAI::Harvester::Record::NamespaceFilter(
-      '*' => $constructor);
+ $constructor = sub { my $x = ""; 
+                      return XML::SAX::Writer->new(Output => \$x);
+                    };
+ $filter = Net::OAI::Record::NamespaceFilter(
+      '*' => $constructor
+   );
  
- my $list = $harvester->listRecords( 
+ $list = $harvester->listRecords( 
      metadataPrefix  => 'oai_dc',
      recordHandler => $filter,
   );
 
- while( my $r = $list->next() ) {
-     my $xmlstringref = $r->recorddata()->result('*');
+ while( $r = $list->next() ) {
+     $xmlstringref = $r->recorddata()->result('*');
      ...
   };
 
-Note:
+
 
 
 =item an already instantiated SAX filter
@@ -98,10 +135,10 @@ Note:
 In this case C<start_document()> and C<end_document()> events are
 E<not> forwarded to the filter. 
 
- open my $fh, ">", $some_file;
+ open $fh, ">", $some_file;
  $builder = XML::SAX::Writer->new(Output => $fh);
  $builder->start_document();
- my $rootEL = { Name => 'collection',
+ $rootEL = { Name => 'collection',
            LocalName => 'collection',
         NamespaceURI => "http://www.loc.gov/MARC21/slim",
               Prefix => "",
@@ -110,14 +147,10 @@ E<not> forwarded to the filter.
  $builder->start_element( $rootEL );
 
  # filter for OAI-Namespace in records: forward all
- my $filter = Net::OAI::Harvester::Record::NamespaceFilter(
+ $filter = Net::OAI::Record::NamespaceFilter(
       'http://www.loc.gov/MARC21/slim' => $builder);
 
- my $harvester = Net::OAI::Harvester->new( [
-     baseURL => ...,
-     ] );
-
- my $list = $harvester->listRecords( 
+ $list = $harvester->listRecords( 
      metadataPrefix  => 'a_strange_one',
      metadataHandler => $filter,
   );
@@ -127,15 +160,24 @@ E<not> forwarded to the filter.
  $builder->end_element( $rootEL );
  $builder->end_document();
  close($fh);
+ # ... process contents of $some_file
 
 =back
 
+Caution: Depending on the namespaces specified, even a handler which
+is freshly instantiated with each OAI record might be fed with more
+than one top-level XML element.
+
 
 =head1 METHODS
 
-
 =head2 new( [%namespaces] )
 
+Creates a Handler suitable as recordHandler or metadataHandler. %namespaces
+has namespace B<URIs> for keys and values according to the four types
+described as above.
+
+
 =cut
 
 sub new {
@@ -151,18 +193,18 @@ sub new {
     $self->{ _performing } = {};
     while ( my ($key, $value) = each %{$self->{ namespaces }} ) {
         if ( ! defined $value ) {   # no handler
-#warn "new(): case 1 for $key";
+            Net::OAI::Harvester::debug( "new(): case 1 for $key" );
           }
         elsif ( ! ref($value) ) {    # class name
-#warn "new(): case 2 for $key";
+            Net::OAI::Harvester::debug( "new(): case 2 for $key: $value");
             Net::OAI::Harvester::_verifyHandler( $value );
           }
         elsif ( ref($value) eq "CODE" ) {    # constructor
-#warn "new(): case 3 for $key";
+            Net::OAI::Harvester::debug( "new(): case 3 for $key");
             # can't verify now
           }
         else {    # active instance
-#warn "new(): case 4 for $key";
+            Net::OAI::Harvester::debug( "new(): case 4 for $key" );
             $self->{ _handlers }->{ $key } = $value;
             $self->{ _performing }->{ $value }--;
           }
@@ -200,7 +242,7 @@ sub STORABLE_thaw {
   my ($obj, $cloning, $serialized, $listref) = @_;
   return if $cloning;
   $obj->{ _result } = $listref;
-#warn "thawed @$listref";
+#carp "thawed @$listref";
 }
 
 
@@ -208,31 +250,32 @@ sub STORABLE_thaw {
 
 sub start_document {
   my ($self, $document) = @_;
-die "start_document()";
-warn "\t\t_activeStack: @{$self->{ _activeStack }}\n";
-warn "\t\t_tagStack: @{$self->{ _tagStack }}\n";
+  carp(<<"XxX");
+unexpected start_document()
+\t_activeStack: @{$self->{ _activeStack }}
+\t_tagStack: @{$self->{ _tagStack }}
+XxX
   $self->SUPER::start_document( $document );
 }
 sub end_document {
   my ($self, $document) = @_;
-die "end_document()";
-warn "\t\t_activeStack: @{$self->{ _activeStack }}\n";
-warn "\t\t_tagStack: @{$self->{ _tagStack }}\n";
+  carp(<<"XxX");
+unexpected end_document()
+\t_activeStack: @{$self->{ _activeStack }}
+\t_tagStack: @{$self->{ _tagStack }}
+XxX
   $self->SUPER::end_document( $document );
 }
 
 sub start_prefix_mapping {
   my ($self, $mapping) = @_;
-#warn "NamespaceFilter: deferred prefix mapping for @{[%$mapping]}\n";
   $self->SUPER::start_prefix_mapping( $mapping ) unless $self->{ _noHandler };
   return if $self->{ _activeStack }->[0];
-#warn ">>>possibly deferred prefix mapping for @{[%$mapping]}\n";
   $self->{ _prefixmap }->{ $mapping->{Prefix} } = $mapping;
   my $activehdl = $self->get_handler();
-die "wrong assumption" unless (! defined $activehdl) or $self->{ _performing }->{ $activehdl };
+  croak ("wrong assumption") unless (! defined $activehdl) or $self->{ _performing }->{ $activehdl };
   my $switched;
   foreach my $hdl ( keys %{$self->{ _performing }} ) {
-#warn "\t-->forwarding prefix mapping @{[%$mapping]}\n\t\tto $hdl at @{$self->{ _tagStack }}\n";
       $self->set_handler( $hdl );
       $self->SUPER::start_prefix_mapping( $mapping );
       $switched = 1;
@@ -244,13 +287,11 @@ sub end_prefix_mapping {
   my ($self, $mapping) = @_;
   $self->SUPER::end_prefix_mapping( $mapping ) unless $self->{ _noHandler };
   return if $self->{ _activeStack }->[0];
-#warn "<<<remove prefix mapping for @{[%$mapping]} at @{$self->{ _tagStack }}\n";
-die "mapping @{[%$mapping]} already removed" unless $self->{ _prefixmap }->{ $mapping->{Prefix} };
+  croak ( "mapping @{[%$mapping]} already removed" ) unless $self->{ _prefixmap }->{ $mapping->{Prefix} };
   my $activehdl = $self->get_handler();   # always undef
-die "wrong assumption" unless (! defined $activehdl) or $self->{ _performing }->{ $activehdl };
+  croak ( "wrong assumption" ) unless (! defined $activehdl) or $self->{ _performing }->{ $activehdl };
   my $switched;
   foreach my $hdl ( keys %{$self->{ _performing }} ) {
-#warn "\t--->forwarding removed mapping to $hdl";
       $self->set_handler( $hdl );
       $self->SUPER::end_prefix_mapping( $mapping );
       $switched = 1;
@@ -261,42 +302,43 @@ die "wrong assumption" unless (! defined $activehdl) or $self->{ _performing }->
 
 sub start_element {
     my ( $self, $element ) = @_;
-#warn "\t((( ".$element->{ Name }." (((";
-#warn "\t\t_activeStack: @{$self->{ _activeStack }}\n";
-#warn "\t\t_tagStack: @{$self->{ _tagStack }}\n";
+#    Net::OAI::Harvester::debug(<<"XxX");
+#\t((( $element->{ Name } (((
+#\t\t_activeStack: @{$self->{ _activeStack }}
+#\t\t_tagStack: @{$self->{ _tagStack }}
+#XxX
     if ( $self->{ _activeStack }->[0] ) {   # handler already set up
       }
     else {
         unless ( $self->{ _tagStack }->[0] ) {      # should be the start of a new record
-#warn "initializing for $element->{Name}\n";
             $self->{ _result } = {};
 # start_document here for all defined handlers?
             my $activehdl = $self->get_handler();   # always undef
-die "handler $activehdl already active" if defined $activehdl;
+            croak( "handler $activehdl already active" ) if defined $activehdl;
             my $switched;
 
             while ( my ($key, $value) = each %{$self->{ namespaces }} ) {
                 $self->{ _result }->{ $key } = undef;
                 my $hdl;
                 if ( ! defined $value ) {   # no handler
-#warn "start_element(): case 1 for $key";
+#                   Net::OAI::Harvester::debug( "start_element(): case 1 for $key" );
                   }
                 elsif ( ! ref($value) ) {    # class name
-#warn "start_element(): case 2 for $key";
+#                   Net::OAI::Harvester::debug( "start_element(): case 2 for $key" );
                     $hdl = $value->new();
                   }
                 elsif ( ref($value) eq "CODE" ) {    # constructor
-#warn "start_element(): case 3 for $key";
+#                   Net::OAI::Harvester::debug( "start_element(): case 3 for $key" );
                     $hdl = &$value();
                     Net::OAI::Harvester::_verifyHandler( $hdl );
                   }
                 else {    # always active instance
-#warn "start_element(): case 4 for $key. Handler is $value";
-# bugfix for XML::SAX::Writer?
+#                   Net::OAI::Harvester::debug( ""start_element(): case 4 for $key. Handler is $value" );
                 $switched = 1;
                 $self->set_handler( $value );
+# Those mapping evends *have* already been forwarded... => Bugfix for XML::SAX::Writer?
                 foreach my $mapping ( values %{$self->{ _prefixmap }} ) {
-#warn "bugfix supply of deferred @{[%$mapping]}";
+#                   Net::OAI::Harvester::debug( "bugfix supply of deferred @{[%$mapping]}" );
                     $self->SUPER::start_prefix_mapping( $mapping )}
                     next;
                   }
@@ -306,10 +348,8 @@ die "handler $activehdl already active" if defined $activehdl;
                 next if $self->{ _performing }->{ $hdl }++;
                 $switched = 1;
                 $self->set_handler( $hdl );
-#warn "dispatching start_document for $hdl";
                 $self->SUPER::start_document({});
                 foreach my $mapping ( values %{$self->{ _prefixmap }} ) {
-#warn "supplying deferred @{[%$mapping]} for $hdl";
                     $self->SUPER::start_prefix_mapping( $mapping )}
               }
             $self->set_handler( $activehdl ) if $switched;
@@ -340,9 +380,11 @@ die "handler $activehdl already active" if defined $activehdl;
 
 sub end_element {
     my ( $self, $element ) = @_;
-#warn "\t))) ".$element->{ Name }." )))";
-#warn "\t\t_activeStack: @{$self->{ _activeStack }}\n";
-#warn "\t\t_tagStack: @{$self->{ _tagStack }}\n";
+#    Net::OAI::Harvester::debug(<<"XxX");
+#\t))) $element->{ Name } )))
+#\t\t_activeStack: @{$self->{ _activeStack }}
+#\t\t_tagStack: @{$self->{ _tagStack }}
+#XxX
     if ( $self->{ _activeStack }->[0] ) {
         unless ( $self->{ _noHandler } ) {
             $self->SUPER::end_element( $element );
@@ -358,39 +400,36 @@ sub end_element {
         pop (@{$self->{ _tagStack }});
       }
     return if $self->{ _tagStack }->[0];
-# create end_document() event here for all handlers?
-#warn "finalizing for $element->{Name}";
     my $activehdl = $self->get_handler();   # always undef
-die "handler $activehdl still active" if defined $activehdl;
+    croak ( "handler $activehdl still active" ) if defined $activehdl;
     my $switched;
     while ( my ($key, $value) = each %{$self->{ namespaces }} ) {
         if ( ! defined $value ) {
-#warn "end_element(): case 1 for $key";
+#           Net::OAI::Harvester::debug( "end_element(): case 1 for $key" );
             $self->{ _result }->{ $key } = "";
           }
         elsif ( my $hdl = $self->{ _handlers }->{ $key } ) {
             if ( ! $self->{ _performing }->{ $hdl } ) {
-                warn "already(?) inactive handler $hdl for $key";
+                carp "already(?) inactive handler $hdl for $key";
                 delete $self->{ _handlers }->{ $key };
                 next;
               }
             elsif ( $self->{ _performing }->{ $hdl } < 0 ) {      # always active handler
-#warn "end_element(): case 4 for $key";
+#               Net::OAI::Harvester::debug( "end_element(): case 4 for $key" );
                 $self->{ _result }->{ $key } = undef;
                 next;
               };
-#warn "end_element(): case 2/3 for $key";
+#           Net::OAI::Harvester::debug( "end_element(): case 2/3 for $key" );
             delete $self->{ _handlers }->{ $key };
             delete $self->{ _performing }->{ $hdl };
             $switched = 1;
             $self->set_handler( $hdl );
 # revoke some stored namespace mappings, too?
             my $result = $self->SUPER::end_document({});
-#warn "dispatching end_document for $hdl yielded $result";
             $self->{ _result }->{ $key } = $result;
           }
         else {
-die " $key not listed as _handler";
+            croak("Assertion failed: $key not listed as _handler");
           };
       };
     $self->set_handler( $activehdl ) if $switched;
